@@ -3,7 +3,6 @@ package http
 import (
 	"context"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"os"
@@ -27,7 +26,7 @@ type DownloaderSuiteBase struct {
 }
 
 func (s *DownloaderSuiteBase) SetUpTest(c *C) {
-	s.tempfile, _ = ioutil.TempFile(os.TempDir(), "aptly-test")
+	s.tempfile, _ = os.CreateTemp(os.TempDir(), "aptly-test")
 	s.l, _ = net.ListenTCP("tcp4", &net.TCPAddr{IP: net.IPv4(127, 0, 0, 1)})
 	s.url = fmt.Sprintf("http://localhost:%d", s.l.Addr().(*net.TCPAddr).Port)
 
@@ -43,7 +42,7 @@ func (s *DownloaderSuiteBase) SetUpTest(c *C) {
 		close(s.ch)
 	}()
 
-	s.progress = console.NewProgress()
+	s.progress = console.NewProgress(false)
 	s.progress.Start()
 
 	s.d = NewDownloader(0, 1, s.progress)
@@ -116,7 +115,7 @@ func (s *DownloaderSuite) TestDownload404(c *C) {
 }
 
 func (s *DownloaderSuite) TestDownloadConnectError(c *C) {
-	c.Assert(s.d.Download(s.ctx, "http://nosuch.host/", s.tempfile.Name()),
+	c.Assert(s.d.Download(s.ctx, "http://nosuch.host.invalid./", s.tempfile.Name()),
 		ErrorMatches, ".*no such host")
 }
 
@@ -151,7 +150,17 @@ func (s *DownloaderSuite) TestGetLength404(c *C) {
 }
 
 func (s *DownloaderSuite) TestGetLengthConnectError(c *C) {
-	_, err := s.d.GetLength(s.ctx, "http://nosuch.host/")
+	_, err := s.d.GetLength(s.ctx, "http://nosuch.host.invalid./")
 
 	c.Assert(err, ErrorMatches, ".*no such host")
 }
+
+func (s *DownloaderSuite) TestContextCancel(c *C) {
+	ctx, cancel := context.WithCancel(s.ctx)
+	s.ctx = ctx
+
+	cancel()
+	_, err := s.d.GetLength(s.ctx, "http://nosuch.host.invalid./")
+
+	c.Assert(err, ErrorMatches, ".*context canceled.*")
+}
