@@ -3,24 +3,27 @@ Test library.
 """
 
 import difflib
+import http.server
 import inspect
 import json
-import subprocess
 import os
 import posixpath
+import pprint
 import re
 import shlex
 import shutil
+import socketserver
 import string
+import subprocess
 import threading
 import urllib.error
 import urllib.parse
 import urllib.request
-import pprint
-import socketserver
-import http.server
 import zlib
 
+from pathlib import Path
+from uuid import uuid4
+
 
 def ungzip_if_required(output):
     if isinstance(output, bytes) and output.startswith(b"\x1f\x8b"):
@@ -100,7 +103,7 @@ class DotFinder(object):
     def find_dot(self, executables):
         for executable in executables:
             try:
-                subprocess.check_output([executable, "-V"], text=True)
+                subprocess.check_output([executable, "-V"], text=True, stderr=subprocess.DEVNULL)
                 return executable
             except Exception:
                 pass
@@ -125,25 +128,15 @@ class BaseTest(object):
     requiresGPG2 = False
     requiresDot = False
     sortOutput = False
+    debugOutput = False
+    EtcdServer = None
 
+    aptlyDir = ".aptly"
+    aptlyConfigFile = ".aptly.conf"
     expectedCode = 0
-    configFile = {
-        "rootDir": "%s/.aptly" % os.environ["HOME"],
-        "downloadConcurrency": 4,
-        "downloadSpeedLimit": 0,
-        "downloadRetries": 5,
-        "databaseOpenAttempts": 10,
-        "architectures": [],
-        "dependencyFollowSuggests": False,
-        "dependencyFollowRecommends": False,
-        "dependencyFollowAllVariants": False,
-        "dependencyFollowSource": False,
-        "gpgDisableVerify": False,
-        "gpgDisableSign": False,
-        "ppaDistributorID": "ubuntu",
-        "ppaCodename": "",
-        "enableMetricsEndpoint": True,
-    }
+    databaseType = ""
+    databaseUrl = ""
+
     configOverride = {}
     environmentOverride = {}
 
@@ -169,26 +162,55 @@ class BaseTest(object):
         try:
             self.run()
             self.check()
+        except Exception as exc:
+            if self.debugOutput:
+                print(f"API log:\n{self.debug_output()}")
+            raise exc
         finally:
             self.teardown()
 
     def prepare_remove_all(self):
-        if os.path.exists(os.path.join(os.environ["HOME"], ".aptly")):
-            shutil.rmtree(os.path.join(os.environ["HOME"], ".aptly"))
-        if os.path.exists(os.path.join(os.environ["HOME"], ".aptly.conf")):
-            os.remove(os.path.join(os.environ["HOME"], ".aptly.conf"))
+        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir)):
+            shutil.rmtree(os.path.join(os.environ["HOME"], self.aptlyDir))
+        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyConfigFile)):
+            os.remove(os.path.join(os.environ["HOME"], self.aptlyConfigFile))
         if os.path.exists(os.path.join(os.environ["HOME"], ".gnupg", "aptlytest.gpg")):
             os.remove(os.path.join(
                 os.environ["HOME"], ".gnupg", "aptlytest.gpg"))
 
     def prepare_default_config(self):
-        cfg = self.configFile.copy()
+        databaseBackend = {
+            "type": self.databaseType,
+            "url": self.databaseUrl,
+        }
+
+        cfg = {
+            "rootDir": f"{os.environ['HOME']}/{self.aptlyDir}",
+            "downloadConcurrency": 4,
+            "downloadSpeedLimit": 0,
+            "downloadRetries": 5,
+            "databaseOpenAttempts": 10,
+            "architectures": [],
+            "dependencyFollowSuggests": False,
+            "dependencyFollowRecommends": False,
+            "dependencyFollowAllVariants": False,
+            "dependencyFollowSource": False,
+            "gpgDisableVerify": False,
+            "gpgDisableSign": False,
+            "ppaDistributorID": "ubuntu",
+            "ppaCodename": "",
+            "enableMetricsEndpoint": True,
+            "logLevel": "debug",
+            "logFormat": "default",
+            "serveInAPIMode": True,
+            "databaseBackend": databaseBackend,
+        }
         if self.requiresGPG1:
             cfg["gpgProvider"] = "gpg1"
         elif self.requiresGPG2:
             cfg["gpgProvider"] = "gpg2"
         cfg.update(**self.configOverride)
-        f = open(os.path.join(os.environ["HOME"], ".aptly.conf"), "w")
+        f = open(os.path.join(os.environ["HOME"], self.aptlyConfigFile), "w")
         f.write(json.dumps(cfg))
         f.close()
 
@@ -210,24 +232,42 @@ class BaseTest(object):
 
     def prepare_fixture(self):
         if self.fixturePool:
-            os.makedirs(os.path.join(os.environ["HOME"], ".aptly"), 0o755)
+            os.makedirs(os.path.join(os.environ["HOME"], self.aptlyDir), 0o755)
             os.symlink(self.fixturePoolDir, os.path.join(
-                os.environ["HOME"], ".aptly", "pool"))
+                os.environ["HOME"], self.aptlyDir, "pool"))
 
         if self.fixturePoolCopy:
-            os.makedirs(os.path.join(os.environ["HOME"], ".aptly"), 0o755)
+            os.makedirs(os.path.join(os.environ["HOME"], self.aptlyDir), 0o755)
             shutil.copytree(self.fixturePoolDir, os.path.join(
-                os.environ["HOME"], ".aptly", "pool"), ignore=shutil.ignore_patterns(".git"))
+                os.environ["HOME"], self.aptlyDir, "pool"), ignore=shutil.ignore_patterns(".git"))
 
-        if self.fixtureDB:
-            shutil.copytree(self.fixtureDBDir, os.path.join(
-                os.environ["HOME"], ".aptly", "db"))
+        if self.databaseType == "etcd":
+            if not os.path.exists("/tmp/aptly-etcd"):
+                self.run_cmd([os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "t13_etcd/install-etcd.sh")])
+
+        if self.fixtureDB and self.databaseType != "etcd":
+            shutil.copytree(self.fixtureDBDir, os.path.join(os.environ["HOME"], self.aptlyDir, "db"))
+
+        if self.databaseType == "etcd":
+            if self.EtcdServer:
+                self.shutdown_etcd()
+
+            # remove existing database
+            if os.path.exists("/tmp/aptly-etcd-data"):
+                shutil.rmtree("/tmp/aptly-etcd-data")
+
+            if self.fixtureDB:
+                print("import etcd")
+                self.run_cmd(["/tmp/aptly-etcd/etcdctl", "--data-dir=/tmp/aptly-etcd-data", "snapshot", "restore", os.path.join(os.environ["HOME"], "etcd.db")])
+
+            print("starting etcd")
+            self.EtcdServer = self._start_process([os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "t13_etcd/start-etcd.sh")], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
         if self.fixtureWebServer:
             self.webServerUrl = self.start_webserver(os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)),
                                                                   self.fixtureWebServer))
 
-        if self.requiresGPG2:
+        if self.requiresGPG2 or self.gpgFinder.gpg2:
             self.run_cmd([
                 self.gpgFinder.gpg2, "--import",
                 os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files") + "/aptly.sec"], expected_code=None)
@@ -235,10 +275,14 @@ class BaseTest(object):
         if self.fixtureGpg:
             self.run_cmd([self.gpgFinder.gpg, "--no-default-keyring", "--trust-model", "always", "--batch", "--keyring", "aptlytest.gpg", "--import"] +
                          [os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", key) for key in self.fixtureGpgKeys])
+            self.run_cmd(["chmod", "400", os.path.join(os.environ["HOME"], ".gnupg/aptlytest.gpg")])
 
         if hasattr(self, "fixtureCmds"):
             for cmd in self.fixtureCmds:
-                self.run_cmd(cmd)
+                output = self.run_cmd(cmd)
+                print("fixture Output:\n")
+                for line in output.split("\n"):
+                    print(f"    {line}")
 
     def sort_lines(self, output):
         return "\n".join(sorted(self.ensure_utf8(output).split("\n")))
@@ -256,31 +300,55 @@ class BaseTest(object):
                 'changes': os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "changes"),
                 'udebs': os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "udebs"),
                 'testfiles': os.path.join(os.path.dirname(inspect.getsourcefile(self.__class__)), self.__class__.__name__),
-                'aptlyroot': os.path.join(os.environ["HOME"], ".aptly"),
+                'aptlyroot': os.path.join(os.environ["HOME"], self.aptlyDir),
             }
             if self.fixtureWebServer:
                 params['url'] = self.webServerUrl
 
             command = string.Template(command).substitute(params)
+            print(f"running command: {command}\n")
             command = shlex.split(command)
 
+        if command[0] == "aptly":
+            aptly_testing_bin = Path(__file__).parent / ".." / "aptly.test"
+            command = [str(aptly_testing_bin), f"-test.coverprofile={Path(self.coverage_dir) / self.__class__.__name__}-{uuid4()}.out", *command[1:]]
+
         environ = os.environ.copy()
         environ["LC_ALL"] = "C"
         environ.update(self.environmentOverride)
         return subprocess.Popen(command, stderr=stderr, stdout=stdout, env=environ)
 
     def run_cmd(self, command, expected_code=0):
-        try:
-            proc = self._start_process(command, stdout=subprocess.PIPE)
-            output, _ = proc.communicate()
-            if expected_code is not None:
-                if proc.returncode != expected_code:
-                    raise Exception("exit code %d != %d (output: %s)" % (
-                        proc.returncode, expected_code, output))
-            return output
-        except Exception as e:
-            raise Exception("Running command %s failed: %s" %
-                            (command, str(e)))
+        proc = self._start_process(command, stdout=subprocess.PIPE)
+        raw_output, _ = proc.communicate()
+
+        raw_output = raw_output.decode("utf-8", errors='replace')
+
+        returncodes = [proc.returncode]
+        is_aptly_command = False
+        if isinstance(command, str):
+            is_aptly_command = command.startswith("aptly")
+
+        if isinstance(command, list):
+            is_aptly_command = command[0] == "aptly"
+
+        if is_aptly_command:
+            # remove the last two rows as go tests always print PASS/FAIL and coverage in those
+            # two lines. This would otherwise fail the tests as they would not match gold
+            matches = re.findall(r"((.|\n)*)EXIT: (\d)\n.*\ncoverage: .*", raw_output)
+            if not matches:
+                raise Exception("no matches found in command output '%s'" % raw_output)
+
+            output, _, returncode = matches[0]
+            returncodes.append(int(returncode))
+        else:
+            output = raw_output
+
+        if expected_code is not None:
+            if expected_code not in returncodes:
+                raise Exception("command expected to return %d, but returned %d: \n%s" % (
+                    expected_code, proc.returncode, raw_output))
+        return output
 
     def gold_processor(self, gold):
         return gold
@@ -307,10 +375,12 @@ class BaseTest(object):
         return s
 
     def check_output(self):
+        gold_file = self.get_gold_filename()
+        print(f"Verifying gold file: {gold_file}")
         try:
             self.verify_match(self.get_gold(), self.output,
                               match_prepare=self.outputMatchPrepare)
-        except:  # noqa: E722
+        except Exception:  # noqa: E722
             if self.captureResults:
                 if self.outputMatchPrepare is not None:
                     self.output = self.outputMatchPrepare(self.output)
@@ -332,12 +402,21 @@ class BaseTest(object):
             else:
                 raise
 
+    def write_file(self, path, content):
+        full_path = os.path.join(os.environ["HOME"], ".aptly", path)
+
+        if not os.path.exists(os.path.dirname(full_path)):
+            os.makedirs(os.path.dirname(full_path), 0o755)
+
+        with open(full_path, "w") as f:
+            f.write(content)
+
     def read_file(self, path, mode=''):
-        with open(os.path.join(os.environ["HOME"], ".aptly", path), "r" + mode) as f:
+        with open(os.path.join(os.environ["HOME"], self.aptlyDir, path), "r" + mode) as f:
             return f.read()
 
     def delete_file(self, path):
-        os.unlink(os.path.join(os.environ["HOME"], ".aptly", path))
+        os.unlink(os.path.join(os.environ["HOME"], self.aptlyDir, path))
 
     def check_file_contents(self, path, gold_name, match_prepare=None, mode='', ensure_utf8=True):
         contents = self.read_file(path, mode=mode)
@@ -367,15 +446,15 @@ class BaseTest(object):
                 raise
 
     def check_exists(self, path):
-        if not os.path.exists(os.path.join(os.environ["HOME"], ".aptly", path)):
+        if not os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir, path)):
             raise Exception("path %s doesn't exist" % (path, ))
 
     def check_not_exists(self, path):
-        if os.path.exists(os.path.join(os.environ["HOME"], ".aptly", path)):
+        if os.path.exists(os.path.join(os.environ["HOME"], self.aptlyDir, path)):
             raise Exception("path %s exists" % (path, ))
 
     def check_file_not_empty(self, path):
-        if os.stat(os.path.join(os.environ["HOME"], ".aptly", path))[6] == 0:
+        if os.stat(os.path.join(os.environ["HOME"], self.aptlyDir, path))[6] == 0:
             raise Exception("file %s is empty" % (path, ))
 
     def check_equal(self, a, b):
@@ -392,11 +471,11 @@ class BaseTest(object):
 
     def check_in(self, item, l):
         if item not in l:
-            raise Exception("item %r not in %r", item, l)
+            raise Exception("expected item: %r\nnot found in: %r" % (item, l))
 
     def check_not_in(self, item, l):
         if item in l:
-            raise Exception("item %r in %r", item, l)
+            raise Exception("unexpected item: %r\n found in: %r" % (item, l))
 
     def check_subset(self, a, b):
         diff = ''
@@ -407,7 +486,7 @@ class BaseTest(object):
                 diff += "wrong value '%s' for key '%s', expected '%s'\n" % (
                     v, k, b[k])
         if diff:
-            raise Exception("content doesn't match:\n" + diff)
+            raise Exception("content subset doesn't match:\n" + diff)
 
     def ensure_utf8(self, a):
         if isinstance(a, bytes):
@@ -423,6 +502,10 @@ class BaseTest(object):
             a = match_prepare(a)
             b = match_prepare(b)
 
+        # strip trailing whitespace and newlines
+        a = a.strip()
+        b = b.strip()
+
         if a != b:
             diff = "".join(difflib.unified_diff(
                 [l + "\n" for l in a.split("\n")], [l + "\n" for l in b.split("\n")]))
@@ -437,7 +520,8 @@ class BaseTest(object):
         self.prepare_fixture()
 
     def teardown(self):
-        pass
+        if self.EtcdServer:
+            self.shutdown_etcd()
 
     def start_webserver(self, directory):
         FileHTTPServerRequestHandler.rootPath = directory
@@ -453,10 +537,18 @@ class BaseTest(object):
     def shutdown(self):
         if hasattr(self, 'webserver'):
             self.shutdown_webserver()
+        if self.EtcdServer:
+            self.shutdown_etcd()
 
     def shutdown_webserver(self):
         self.webserver.shutdown()
 
+    def shutdown_etcd(self):
+        print("stopping etcd")
+        self.EtcdServer.terminate()
+        self.EtcdServer.wait()
+        self.EtcdServer = None
+
     @classmethod
     def shutdown_class(cls):
         pass
