@@ -4,21 +4,31 @@ package utils
 import (
 	"fmt"
 	"os"
+	"strings"
 
 	"golang.org/x/sys/unix"
 )
 
 // DirIsAccessible verifies that directory exists and is accessible
 func DirIsAccessible(filename string) error {
-	_, err := os.Stat(filename)
+	fileStat, err := os.Stat(filename)
 	if err != nil {
 		if !os.IsNotExist(err) {
 			return fmt.Errorf("error checking directory '%s': %s", filename, err)
 		}
 	} else {
-		if unix.Access(filename, unix.W_OK) != nil {
+		if fileStat.Mode().Perm() == 0000 || unix.Access(filename, unix.W_OK) != nil {
 			return fmt.Errorf("'%s' is inaccessible, check access rights", filename)
 		}
 	}
 	return nil
 }
+
+// Remove leading '/', remove '..', '$' and '`'
+func SanitizePath(path string) (result string) {
+	result = strings.Replace(path, "..", "", -1)
+	result = strings.Replace(result, "$", "", -1)
+	result = strings.Replace(result, "`", "", -1)
+	result = strings.TrimLeft(result, "/")
+	return
+}
